property pSprite, pSpriteNum, pActive, pVector, pVectorLength, pDestination, pOrigin, pQTSpeed, pInited, pPosition, pDirection, pDistance, pPointH, pPointV, pBroadcast

on getBehaviorDescription me
  return "CONSTRAIN TO LINE" & RETURN & RETURN & "[Advanced]" & RETURN & RETURN & "Creates a slider constrained to a straight line." & RETURN & RETURN & "Place the sprite at a position indicating the lowest value of the slider (typically the leftmost position) and use the behavior parameters to determine the direction the slider may be dragged." & RETURN & RETURN & "There are two methods to determine the direction of the slider:" & RETURN & RETURN & "The simplest is to select a direction (such as 'right') and a distance (in pixels) the slider can move. " & "Presets are available for each 45 degree angle." & RETURN & RETURN & "The other method is to choose 'point' and a coordinate for the sprite to move toward, allowing movements on other angles. " & "The initial position for the sprite to appear along the path is set by a number from 0 to 1 (default is 0.5, halfway between the maximum and minimum positions on the path)." & RETURN & RETURN & "The behavior broadcasts its current setting (always a number between 0 and 1) if the broadcast flag is set to anything but 0. " & "To intercept the broadcast, place an mConstrainedValue handler in a behavior on the target sprite." & RETURN & RETURN & "Example:" & RETURN & "  on mConstrainedValue(me, vSlider, vValue)" & RETURN & RETURN & "vSlider is the sprite number of the slider sending the message (enabling tests if there is more than one slider operating); vValue is the slider's current position." & RETURN & RETURN & "Use the mSetConstrainPos method to set the position of the slider from another handler. " & "For example," & RETURN & RETURN & "  sendSprite(sliderSpriteNum, #mSetConstrainPos, 0.66, FALSE)" & RETURN & RETURN & "would set the slider to a position two-thirds of the way between minimum and maximum positions without sending a new message to the broadcast sprite." & RETURN & RETURN & "The mIncrementConstrainPos handler can move the slider a set distance. " & "For example," & RETURN & RETURN & "  sendSprite(sliderSprite, #mIncrementConstrainPos, 0.05)" & RETURN & RETURN & "would move the sprite 1/20th of the distance from the minimum position to the maximum position. " & "Negative values move toward the minimum position." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "bitmap, Flash, QuickTime" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Position slider first appears on path" & RETURN & "* Direction sprite will slide" & RETURN & "* Distance sprite moves (used for relative direction)" & RETURN & "* Vertical point coordinate (used for specific point)" & RETURN & "* Horizontal point coordinate (used for specific point)" & RETURN & "* Broadcast flag for slider position"
end

on getBehaviorTooltip me
  return "[ADVANCED]" & RETURN & RETURN & "Makes a sprite draggable along a straight line." & RETURN & RETURN & "Place a sprite at one end of the line it should be dragged along, set parameters to determine the other end of the line, and where the draggable sprite should first appear. " & "The behavior can report its position along the line as a message to another sprite."
end

on beginSprite me
  mInitialize(me)
end

on prepareFrame me
  mUpdate(me)
end

on mouseDown me
  pActive = 1
  if sprite(pBroadcast).member.type = #quickTimeMedia then
    pQTSpeed = sprite(pBroadcast).movieRate
    sprite(pBroadcast).movieRate = 0
  end if
end

on mInitialize me
  pSpriteNum = me.spriteNum
  pSprite = sprite(pSpriteNum)
  pActive = 0
  pOrigin = pSprite.loc
  vStage = (the stage).rect
  vDiagonal = integer(pDistance * sqrt(2.0) / 2)
  case pDirection of
    "Point":
      pDestination = point(pPointH, pPointV)
    "Up":
      pDestination = point(pSprite.locH, max(0, pSprite.locV - pDistance))
    "Down":
      pDestination = point(pSprite.locH, min(vStage.height, pSprite.locV + pDistance))
    "Left":
      pDestination = point(max(0, pSprite.locH - pDistance), pSprite.locV)
    "Right":
      pDestination = point(min(vStage.width, pSprite.locH + pDistance), pSprite.locV)
    "Up Left":
      pDestination = point(max(0, pSprite.locH - vDiagonal), max(0, pSprite.locV - vDiagonal))
    "Up Right":
      pDestination = point(min(vStage.width, pSprite.locH + pDistance), max(0, pSprite.locV - vDiagonal))
    "Down Left":
      pDestination = point(max(0, pSprite.locH - vDiagonal), min(vStage.width, pSprite.locV + vDiagonal))
    "Down Right":
      pDestination = point(min(vStage.width, pSprite.locH + pDistance), min(vStage.width, pSprite.locV + vDiagonal))
  end case
  pVector = pDestination - pOrigin
  pVectorLength = mVectorLength(pVector)
  if sprite(pBroadcast).member.type = #quickTimeMedia then
    pQTSpeed = sprite(pBroadcast).movieRate
  else
    pQTSpeed = 0
  end if
  vMember = pSprite.member
  case vMember.type of
    #animGif, #flash, #quickTimeMedia, #digitalVideo, #vectorShape:
      if vMember.directToStage then
        alert("Sprite" && pSpriteNum & ": Direct To Stage media may cause" && "playback problems with the 'Constrain to Line' behavior.")
      end if
  end case
  mSetConstrainPos(me, pPosition, 1)
  pInited = 0
end

on mUpdate me
  if not pInited then
    mSetConstrainPos(me, pPosition, 1)
    pInited = 1
  end if
  if pActive then
    if the mouseUp then
      pActive = 0
      if sprite(pBroadcast).member.type = #quickTimeMedia then
        sprite(pBroadcast).movieRate = pQTSpeed
      end if
    else
      vDiff = pSprite.loc - the mouseLoc
      vDist = mVectorLength(vDiff)
      if vDist > 0 then
        vPercent = min(1.0, vDist / pVectorLength) / 2
        vAhead = min(pPosition + vPercent, 1.0)
        vBehind = max(pPosition - vPercent, 0.0)
        vAheadPoint = pOrigin + mScaleVector(pVector, vAhead)
        vBehindPoint = pOrigin + mScaleVector(pVector, vBehind)
        vAheadDist = mVectorLength(vAheadPoint - the mouseLoc)
        vBehindDist = mVectorLength(vBehindPoint - the mouseLoc)
        vCompare = vAheadDist - vBehindDist
        if vCompare <> 0.0 then
          if vCompare < 0 then
            if vDist > vAheadDist then
              mSetConstrainPos(me, vAhead, 1)
            end if
          else
            if vDist > vBehindDist then
              mSetConstrainPos(me, vBehind, 1)
            end if
          end if
        end if
      end if
    end if
  end if
end

on mScaleVector vVector, vScale
  return vVector * integer(1000 * vScale) / 1000
end

on mVectorLength vVector
  vSquare = (vVector.locH * vVector.locH) + (vVector.locV * vVector.locV)
  return sqrt(float(vSquare))
end

on mIncrementConstrainPos me, vIncrement
  mSetConstrainPos(me, pPosition + vIncrement, 1)
end

on mSetConstrainPos me, vPosition, vUpdate
  pPosition = max(min(1.0, vPosition), 0.0)
  pSprite.loc = pOrigin + mScaleVector(pVector, pPosition)
  if pBroadcast then
    if vUpdate then
      sendSprite(pBroadcast, #mConstrainedValue, pSpriteNum, pPosition)
    end if
  end if
end

on isOKToAttach me, aSpriteType, aSpriteNum
  return aSpriteType = #graphic
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  vStage = (the stage).rect
  vStageMeasure = max(vStage.height, vStage.width)
  vPDList = [:]
  setaProp(vPDList, #pDirection, [#comment: "Constraint direction (relative to current position)", #format: #string, #default: "Right", #range: mDirList()])
  setaProp(vPDList, #pDistance, [#comment: "Distance (in pixels)", #format: #integer, #default: min(100, vStageMeasure), #range: [#min: 10, #max: vStageMeasure]])
  setaProp(vPDList, #pPosition, [#comment: "Initial position on line (from 0 to 1)", #format: #float, #default: 0.5, #range: [#min: 0.0, #max: 1.0]])
  setaProp(vPDList, #pPointH, [#comment: "Point Horizontal Value", #format: #integer, #default: vStage.width / 2, #range: [#min: 0, #max: vStage.width]])
  setaProp(vPDList, #pPointV, [#comment: "Point Vertical Value", #format: #integer, #default: vStage.height / 2, #range: [#min: 0, #max: vStage.height]])
  setaProp(vPDList, #pBroadcast, [#comment: "Broadcast position to sprite (0 = no broadcast)", #format: #integer, #default: 0, #range: [#min: 0, #max: the lastChannel]])
  return vPDList
end

on mDirList
  return ["Point", "Up", "Down", "Left", "Right", "Up Left", "Up Right", "Down Left", "Down Right"]
end
